﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Globalization;

namespace howto_raise_events
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The BankAccount object this program manages.
        private BankAccount TheBankAccount;

        // Initialize the BankAccount object.
        private void Form1_Load(object sender, EventArgs e)
        {
            TheBankAccount = new BankAccount();
            TheBankAccount.Balance = 100m;
            txtBalance.Text = TheBankAccount.Balance.ToString("C");

            // Subscribe to the Overdrawn event.
            TheBankAccount.Overdrawn += Account_Overdrawn;
        }

        // Add money to the account.
        private void btnCredit_Click(object sender, EventArgs e)
        {
            // Get the amount.
            decimal amount;
            if (!decimal.TryParse(txtAmount.Text, NumberStyles.Currency,
                CultureInfo.CurrentCulture, out amount))
            {
                MessageBox.Show("The amount must be a currency value.",
                    "Invalid Amount", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                txtAmount.Focus();
            }

            // Post the credit.
            try
            {
                TheBankAccount.Credit(amount);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            // Display the new balance.
            txtBalance.Text = TheBankAccount.Balance.ToString("C");
        }

        // Remove money from the account.
        private void btnDebit_Click(object sender, EventArgs e)
        {
            // Get the amount.
            decimal amount;
            if (!decimal.TryParse(txtAmount.Text, NumberStyles.Currency,
                CultureInfo.CurrentCulture, out amount))
            {
                MessageBox.Show("The amount must be a currency value.",
                    "Invalid Amount", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                txtAmount.Focus();
            }

            // Post the debit.
            try
            {
                TheBankAccount.Debit(amount);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            // Display the new balance.
            txtBalance.Text = TheBankAccount.Balance.ToString("C");
        }

        // Handle the account's Overdrawn event.
        private void Account_Overdrawn(object sender, OverdrawnArgs e)
        {
            // Get the account.
            BankAccount account = sender as BankAccount;

            // Ask the user whether to allow this.
            if (MessageBox.Show("Insufficient funds.\n\n    Current balance: " +
                account.Balance.ToString("C") + "\n    Debit amount: " +
                e.DebitAmount.ToString("C") + "\n\n" +
                "Do you want to allow this transaction anyway?",
                "Allow?", MessageBoxButtons.YesNo, MessageBoxIcon.Question)
                    == DialogResult.Yes)
            {
                // Allow the transaction anyway.
                e.Allow = true;
            }
        }
    }
}
